@@ -5,8 +5,8 @@
 #
 
 # environment variables
+from redisbench_admin.run.args import TRIGGERING_ENV
 from redisbench_admin.run.common import get_start_time_vars
-from redisbench_admin.run_remote.args import TRIGGERING_ENV
 from redisbench_admin.utils.remote import (
     PERFORMANCE_RTS_HOST,
     PERFORMANCE_RTS_PORT,
@@ -41,8 +41,10 @@ def create_compare_arguments(parser):
     parser.add_argument("--deployment_type", type=str, default="oss-standalone")
     parser.add_argument("--metric_name", type=str, default="Tests.Overall.rps")
     parser.add_argument("--metric_mode", type=str, default="higher-better")
-    parser.add_argument("--baseline-branch", type=str, default=None, required=True)
-    parser.add_argument("--comparison-branch", type=str, default=None, required=True)
+    parser.add_argument("--baseline-branch", type=str, default=None, required=False)
+    parser.add_argument("--baseline-tag", type=str, default=None, required=False)
+    parser.add_argument("--comparison-branch", type=str, default=None, required=False)
+    parser.add_argument("--comparison-tag", type=str, default=None, required=False)
     parser.add_argument("--print-regressions-only", type=bool, default=False)
     parser.add_argument(
         "--regressions-percent-lower-limit",
