@@ -18,7 +18,6 @@ from redisbench_admin.run.redis_benchmark.redis_benchmark import (
 from redisbench_admin.run.tsbs_run_queries_redistimeseries.tsbs_run_queries_redistimeseries import (
     extract_tsbs_extra_links,
 )
-from redisbench_admin.run_remote.consts import private_key
 from redisbench_admin.utils.benchmark_config import results_dict_kpi_check
 from redisbench_admin.utils.redisgraph_benchmark_go import (
     setup_remote_benchmark_tool_redisgraph_benchmark_go,
@@ -37,17 +36,17 @@ def absoluteFilePaths(directory):
             yield os.path.abspath(os.path.join(dirpath, f))
 
 
-def benchmark_tools_sanity_check(args, benchmark_tool):
+def benchmark_tools_sanity_check(allowed_tools, benchmark_tool):
     if benchmark_tool is not None:
         logging.info("Detected benchmark config tool {}".format(benchmark_tool))
     else:
         raise Exception(
             "Unable to detect benchmark tool within 'clientconfig' section. Aborting..."
         )
-    if benchmark_tool not in args.allowed_tools.split(","):
+    if benchmark_tool not in allowed_tools.split(","):
         raise Exception(
             "Benchmark tool {} not in the allowed tools list [{}]. Aborting...".format(
-                benchmark_tool, args.allowed_tools
+                benchmark_tool, allowed_tools
             )
         )
 
@@ -61,6 +60,12 @@ def remote_tool_pre_bench_step(
     benchmark_tool,
     client_public_ip,
     username,
+    benchmark_tool_source,
+    config_key,
+    os_str,
+    arch_str,
+    client_ssh_port,
+    private_key,
 ):
     logging.info("Settting up remote tool {} requirements".format(benchmark_tool))
     if benchmark_tool == "redisgraph-benchmark-go":
@@ -69,12 +74,15 @@ def remote_tool_pre_bench_step(
             username,
             private_key,
             redisbenchmark_go_link,
+            client_ssh_port,
         )
     if "ycsb" in benchmark_tool:
         setup_remote_benchmark_tool_ycsb_redisearch(
             client_public_ip,
             username,
             private_key,
+            benchmark_tool_source,
+            client_ssh_port,
         )
 
     if "ftsb_" in benchmark_tool:
@@ -88,7 +96,7 @@ def remote_tool_pre_bench_step(
                 queries_file_link, remote_tool_link, tool_link
             )
         )
-
+        remote_input_file = "/tmp/input.data"
         setup_remote_benchmark_tool_requirements_ftsb(
             client_public_ip,
             username,
@@ -96,15 +104,15 @@ def remote_tool_pre_bench_step(
             tool_link,
             queries_file_link,
             remote_tool_link,
+            remote_input_file,
+            client_ssh_port,
         )
 
     if "tsbs_" in benchmark_tool:
-        (
-            queries_file_link,
-            remote_tool_link,
-            tool_link,
-        ) = extract_tsbs_extra_links(benchmark_config, benchmark_tool)
-
+        (queries_file_link, remote_tool_link, tool_link,) = extract_tsbs_extra_links(
+            benchmark_config, benchmark_tool, config_key, os_str, arch_str
+        )
+        remote_input_file = "/tmp/input.data"
         setup_remote_benchmark_tool_requirements_tsbs(
             client_public_ip,
             username,
@@ -112,6 +120,8 @@ def remote_tool_pre_bench_step(
             tool_link,
             queries_file_link,
             remote_tool_link,
+            remote_input_file,
+            client_ssh_port,
         )
     if "aibench_" in benchmark_tool:
         (
@@ -119,7 +129,7 @@ def remote_tool_pre_bench_step(
             remote_tool_link,
             tool_link,
         ) = extract_aibench_extra_links(benchmark_config, benchmark_tool)
-
+        remote_input_file = "/tmp/input.data"
         setup_remote_benchmark_tool_requirements_tsbs(
             client_public_ip,
             username,
@@ -127,6 +137,8 @@ def remote_tool_pre_bench_step(
             tool_link,
             queries_file_link,
             remote_tool_link,
+            remote_input_file,
+            client_ssh_port,
         )
     if benchmark_min_tool_version is not None and benchmark_tool == "redis-benchmark":
         redis_benchmark_ensure_min_version_remote(
@@ -138,6 +150,7 @@ def remote_tool_pre_bench_step(
             client_public_ip,
             username,
             private_key,
+            client_ssh_port,
         )
     logging.info("Finished up remote tool {} requirements".format(benchmark_tool))
 
@@ -149,14 +162,17 @@ def setup_remote_benchmark_tool_requirements_ftsb(
     tool_link,
     queries_file_link,
     remote_tool_link,
-    remote_input_file="/tmp/input.data",
+    remote_input_file,
+    client_ssh_port,
 ):
     commands = [
         "wget {} -q -O {}".format(tool_link, remote_tool_link),
         "wget {} -q -O {}".format(queries_file_link, remote_input_file),
         "chmod 755 {}".format(remote_tool_link),
     ]
-    execute_remote_commands(client_public_ip, username, private_key, commands)
+    execute_remote_commands(
+        client_public_ip, username, private_key, commands, client_ssh_port
+    )
 
 
 def setup_remote_benchmark_tool_requirements_tsbs(
@@ -166,23 +182,28 @@ def setup_remote_benchmark_tool_requirements_tsbs(
     tool_link,
     queries_file_link,
     remote_tool_link,
-    remote_input_file="/tmp/input.data",
+    remote_input_file,
+    client_ssh_port,
 ):
     commands = [
         "wget {} -q -O {}".format(tool_link, remote_tool_link),
         "wget {} -q -O {}".format(queries_file_link, remote_input_file),
         "chmod 755 {}".format(remote_tool_link),
     ]
-    execute_remote_commands(client_public_ip, username, private_key, commands)
+    execute_remote_commands(
+        client_public_ip, username, private_key, commands, client_ssh_port
+    )
 
 
 def extract_artifact_version_remote(
-    server_public_ip, server_public_port, username, private_key
+    server_public_ip, server_public_port, username, private_key, client_ssh_port
 ):
     commands = [
         "redis-cli -h {} -p {} info modules".format("localhost", server_public_port),
     ]
-    res = execute_remote_commands(server_public_ip, username, private_key, commands)
+    res = execute_remote_commands(
+        server_public_ip, username, private_key, commands, client_ssh_port
+    )
     recv_exit_status, stdout, stderr = res[0]
     print(stdout)
     module_name, version = extract_module_semver_from_info_modules_cmd(stdout)
