@@ -36,8 +36,14 @@ def parse_exporter_timemetric_definition(
 
 
 def parse_exporter_timemetric(metric_path: str, results_dict: dict):
-    jsonpath_expr = parse(metric_path)
-    datapoints_timestamp = int(jsonpath_expr.find(results_dict)[0].value)
+    datapoints_timestamp = None
+    try:
+        jsonpath_expr = parse(metric_path)
+        datapoints_timestamp = int(jsonpath_expr.find(results_dict)[0].value)
+    except Exception as e:
+        logging.error(
+            "Unable to parse time-metric {}. Error: {}".format(metric_path, e.__str__())
+        )
     return datapoints_timestamp
 
 
@@ -166,7 +172,10 @@ def merge_default_and_specific_properties_dict_type(
 def extract_redis_dbconfig_parameters(benchmark_config, dbconfig_keyname):
     redis_configuration_parameters = {}
     dataset_load_timeout_secs = 120
+    dataset_name = None
+    dbconfig_present = False
     if dbconfig_keyname in benchmark_config:
+        dbconfig_present = True
         for k in benchmark_config[dbconfig_keyname]:
             if "configuration-parameters" in k:
                 cp = k["configuration-parameters"]
@@ -175,8 +184,15 @@ def extract_redis_dbconfig_parameters(benchmark_config, dbconfig_keyname):
                         redis_configuration_parameters[k] = v
             if "dataset_load_timeout_secs" in k:
                 dataset_load_timeout_secs = k["dataset_load_timeout_secs"]
+            if "dataset_name" in k:
+                dataset_name = k["dataset_name"]
 
-    return redis_configuration_parameters, dataset_load_timeout_secs
+    return (
+        dbconfig_present,
+        dataset_name,
+        redis_configuration_parameters,
+        dataset_load_timeout_secs,
+    )
 
 
 def process_default_yaml_properties_file(
@@ -237,7 +253,30 @@ def extract_exporter_metrics(default_config):
     return default_metrics, exporter_timemetric_path
 
 
-def extract_benchmark_tool_settings(benchmark_config):
+def extract_benchmark_type_from_config(
+    benchmark_config,
+    config_key="clientconfig",
+    benchmark_type_key="benchmark_type",
+    default_benchmark_type="mixed",
+):
+    benchmark_config_present = False
+    benchmark_type = None
+    if config_key in benchmark_config:
+        benchmark_config_present = True
+        for entry in benchmark_config[config_key]:
+            if benchmark_type_key in entry:
+                benchmark_type = entry[benchmark_type_key]
+    if benchmark_type is None:
+        logging.info(
+            "Given the '{}' info was not present on {} we will assume the most inclusive default: '{}'".format(
+                benchmark_type_key, config_key, default_benchmark_type
+            )
+        )
+        benchmark_type = default_benchmark_type
+    return benchmark_config_present, benchmark_type
+
+
+def extract_benchmark_tool_settings(benchmark_config, config_key="clientconfig"):
     benchmark_tool = None
     benchmark_tool_source = None
     benchmark_tool_source_inner_path = None
@@ -245,8 +284,8 @@ def extract_benchmark_tool_settings(benchmark_config):
     benchmark_min_tool_version_major = None
     benchmark_min_tool_version_minor = None
     benchmark_min_tool_version_patch = None
-    benchmark_tool_property_map = benchmark_config["clientconfig"]
-    for entry in benchmark_config["clientconfig"]:
+    benchmark_tool_property_map = benchmark_config[config_key]
+    for entry in benchmark_config[config_key]:
         if "tool" in entry:
             benchmark_tool = entry["tool"]
         if "tool_source" in entry:
@@ -283,7 +322,7 @@ def extract_benchmark_tool_settings(benchmark_config):
 
 
 def get_testfiles_to_process(args):
-    defaults_filename = "defaults.yml"
+    defaults_filename = args.defaults_filename
     if args.test == "":
         files = pathlib.Path().glob("*.yml")
         files = [str(x) for x in files]
