@@ -1,12 +1,17 @@
+#  Apache License Version 2.0
+#
+#  Copyright (c) 2021., Redis Labs Modules
+#  All rights reserved.
+#
+
 import logging
 import os
-import subprocess
-import time
 from shutil import copyfile
 
-import redis
 import wget
 
+from redisbench_admin.environments.oss_cluster import get_cluster_dbfilename
+
 
 def check_dataset_local_requirements(
     benchmark_config,
@@ -14,6 +19,8 @@ def check_dataset_local_requirements(
     dirname=None,
     datasets_localtemp_dir="./datasets",
     dbconfig_keyname="dbconfig",
+    number_primaries=1,
+    is_cluster=False,
 ):
     dataset = None
     full_path = None
@@ -26,11 +33,23 @@ def check_dataset_local_requirements(
             full_path = check_if_needs_remote_fetch(
                 dataset, datasets_localtemp_dir, dirname
             )
-            tmp_path = "{}/dump.rdb".format(redis_dbdir)
-            logging.info(
-                "Copying rdb from {} to {}/dump.rdb".format(full_path, redis_dbdir)
-            )
-            copyfile(full_path, tmp_path)
+
+            if is_cluster is False:
+                tmp_path = "{}/dump.rdb".format(redis_dbdir)
+                logging.info("Copying rdb from {} to {}".format(full_path, tmp_path))
+                copyfile(full_path, tmp_path)
+            else:
+                start_port = 6379
+                for primary_number in range(number_primaries):
+                    primary_port = start_port + primary_number
+                    tmp_path = "{}/{}".format(
+                        redis_dbdir, get_cluster_dbfilename(primary_port)
+                    )
+                    logging.info(
+                        "Copying rdb from {} to {}".format(full_path, tmp_path)
+                    )
+                    copyfile(full_path, tmp_path)
+
     return dataset, full_path, tmp_path
 
 
@@ -62,80 +81,6 @@ def check_if_needs_remote_fetch(
     return full_path
 
 
-def wait_for_conn(conn, retries=20, command="PING", should_be=True):
-    """Wait until a given Redis connection is ready"""
-    result = False
-    while retries > 0 and result is False:
-        try:
-            if conn.execute_command(command) == should_be:
-                result = True
-        except redis.exceptions.BusyLoadingError:
-            time.sleep(0.1)  # give extra 100msec in case of RDB loading
-        except redis.ConnectionError as err:
-            str(err)
-        except redis.ResponseError as err:
-            err1 = str(err)
-            if not err1.startswith("DENIED"):
-                raise
-        time.sleep(0.1)
-        retries -= 1
-        logging.debug("Waiting for Redis")
-    return result
-
-
-def spin_up_local_redis(
-    dbdir,
-    port,
-    local_module_file,
-    configuration_parameters=None,
-):
-    command = generate_standalone_redis_server_args(
-        dbdir, local_module_file, port, configuration_parameters
-    )
-
-    logging.info(
-        "Running local redis-server with the following args: {}".format(
-            " ".join(command)
-        )
-    )
-    redis_process = subprocess.Popen(command)
-    result = wait_for_conn(redis.StrictRedis(port=port))
-    if result is True:
-        logging.info("Redis available")
-    return redis_process
-
-
-def generate_standalone_redis_server_args(
-    dbdir, local_module_file, port, configuration_parameters=None
-):
-    # start redis-server
-    command = [
-        "redis-server",
-        "--save",
-        '""',
-        "--port",
-        "{}".format(port),
-        "--dir",
-        dbdir,
-    ]
-    if configuration_parameters is not None:
-        for parameter, parameter_value in configuration_parameters.items():
-            command.extend(
-                [
-                    "--{}".format(parameter),
-                    parameter_value,
-                ]
-            )
-    if local_module_file is not None:
-        command.extend(
-            [
-                "--loadmodule",
-                os.path.abspath(local_module_file),
-            ]
-        )
-    return command
-
-
 def is_process_alive(process):
     if not process:
         return False
@@ -150,9 +95,11 @@ def get_local_run_full_filename(
     start_time_str,
     github_branch,
     test_name,
+    setup_name,
 ):
     benchmark_output_filename = (
-        "{start_time_str}-{github_branch}-{test_name}.json".format(
+        "{setup_name}-{start_time_str}-{github_branch}-{test_name}.json".format(
+            setup_name=setup_name,
             start_time_str=start_time_str,
             github_branch=github_branch,
             test_name=test_name,
