@@ -14,11 +14,12 @@ import git
 import paramiko
 import pysftp
 import redis
-import redistimeseries.client as client
 from git import Repo
 from jsonpath_ng import parse
 from python_terraform import Terraform
 
+from redisbench_admin.environments.oss_cluster import get_cluster_dbfilename
+from redisbench_admin.run.metrics import extract_results_table
 from redisbench_admin.utils.local import check_dataset_local_requirements
 from redisbench_admin.utils.utils import (
     get_ts_metric_name,
@@ -33,6 +34,11 @@ PERFORMANCE_RTS_AUTH = os.getenv("PERFORMANCE_RTS_AUTH", None)
 PERFORMANCE_RTS_USER = os.getenv("PERFORMANCE_RTS_USER", None)
 PERFORMANCE_RTS_HOST = os.getenv("PERFORMANCE_RTS_HOST", "localhost")
 PERFORMANCE_RTS_PORT = os.getenv("PERFORMANCE_RTS_PORT", 6379)
+# DB used to authenticate ( read-only/non-dangerous access only )
+REDIS_AUTH_SERVER_HOST = os.getenv("REDIS_AUTH_SERVER_HOST", "localhost")
+REDIS_AUTH_SERVER_PORT = int(os.getenv("REDIS_AUTH_SERVER_PORT", "6380"))
+REDIS_HEALTH_CHECK_INTERVAL = int(os.getenv("REDIS_HEALTH_CHECK_INTERVAL", "15"))
+REDIS_SOCKET_TIMEOUT = int(os.getenv("REDIS_SOCKET_TIMEOUT", "300"))
 TERRAFORM_BIN_PATH = os.getenv("TERRAFORM_BIN_PATH", "terraform")
 
 
@@ -49,7 +55,13 @@ def view_bar_simple(a, b):
 
 
 def copy_file_to_remote_setup(
-    server_public_ip, username, private_key, local_file, remote_file, dirname=None
+    server_public_ip,
+    username,
+    private_key,
+    local_file,
+    remote_file,
+    dirname=None,
+    port=22,
 ):
     full_local_path = local_file
     if dirname is not None:
@@ -70,6 +82,7 @@ def copy_file_to_remote_setup(
             username=username,
             private_key=private_key,
             cnopts=cnopts,
+            port=port,
         )
         srv.put(full_local_path, remote_file, callback=view_bar_simple)
         srv.close()
@@ -111,23 +124,36 @@ def fetch_file_from_remote_setup(
     )
 
 
-def execute_remote_commands(server_public_ip, username, private_key, commands):
+def execute_remote_commands(
+    server_public_ip, username, private_key, commands, port, get_pty=False
+):
     res = []
+    c = connect_remote_ssh(port, private_key, server_public_ip, username)
+    for command in commands:
+        logging.info('Executing remote command "{}"'.format(command))
+        stdin, stdout, stderr = c.exec_command(command, get_pty=get_pty)
+        recv_exit_status = stdout.channel.recv_exit_status()  # status is 0
+        stdout = stdout.readlines()
+        stderr = stderr.readlines()
+        if recv_exit_status != 0:
+            logging.warning(
+                "Exit status: {} for command {}.\n\tSTDERR: {}\n\tSTDOUT: {}".format(
+                    recv_exit_status, command, stdout, stderr
+                )
+            )
+        res.append([recv_exit_status, stdout, stderr])
+    c.close()
+    return res
+
+
+def connect_remote_ssh(port, private_key, server_public_ip, username):
     k = paramiko.RSAKey.from_private_key_file(private_key)
     c = paramiko.SSHClient()
     c.set_missing_host_key_policy(paramiko.AutoAddPolicy())
     logging.info("Connecting to remote server {}".format(server_public_ip))
-    c.connect(hostname=server_public_ip, username=username, pkey=k)
+    c.connect(hostname=server_public_ip, port=port, username=username, pkey=k)
     logging.info("Connected to remote server {}".format(server_public_ip))
-    for command in commands:
-        logging.info('Executing remote command "{}"'.format(command))
-        stdin, stdout, stderr = c.exec_command(command)
-        recv_exit_status = stdout.channel.recv_exit_status()  # status is 0
-        stdout = stdout.readlines()
-        stderr = stderr.readlines()
-        res.append([recv_exit_status, stdout, stderr])
-    c.close()
-    return res
+    return c
 
 
 def check_dataset_remote_requirements(
@@ -135,10 +161,12 @@ def check_dataset_remote_requirements(
     server_public_ip,
     username,
     private_key,
-    remote_dataset_file,
+    remote_dataset_folder,
     dirname,
     number_primaries,
     is_cluster,
+    start_port,
+    db_ssh_port=22,
 ):
     res = True
     dataset, fullpath, tmppath = check_dataset_local_requirements(
@@ -148,7 +176,7 @@ def check_dataset_remote_requirements(
         "./datasets",
         "dbconfig",
         number_primaries,
-        is_cluster,
+        False,
     )
     if dataset is not None:
         logging.info(
@@ -156,6 +184,18 @@ def check_dataset_remote_requirements(
                 dataset
             )
         )
+        if is_cluster:
+            primary_port = start_port
+            remote_dataset_file = "{}/{}".format(
+                remote_dataset_folder, get_cluster_dbfilename(primary_port)
+            )
+        else:
+            remote_dataset_file = "{}/dump.rdb".format(remote_dataset_folder)
+        logging.info(
+            "Copying rdb from {} to remote machine(eip={}) into :{}".format(
+                fullpath, server_public_ip, remote_dataset_file
+            )
+        )
         res = copy_file_to_remote_setup(
             server_public_ip,
             username,
@@ -163,7 +203,32 @@ def check_dataset_remote_requirements(
             fullpath,
             remote_dataset_file,
             None,
+            db_ssh_port,
         )
+        if is_cluster:
+            commands = []
+            for master_shard_id in range(2, number_primaries + 1):
+                primary_port = master_shard_id + start_port - 1
+                second_forward_remote_dataset_file = "{}/{}".format(
+                    remote_dataset_folder, get_cluster_dbfilename(primary_port)
+                )
+                logging.info(
+                    "For primary #{}, reusing the already present rdb in {} and duplicating it into :{}".format(
+                        master_shard_id,
+                        remote_dataset_file,
+                        second_forward_remote_dataset_file,
+                    )
+                )
+                # start redis-server
+                commands.append(
+                    "cp {} {}".format(
+                        remote_dataset_file, second_forward_remote_dataset_file
+                    )
+                )
+            execute_remote_commands(
+                server_public_ip, username, private_key, commands, db_ssh_port
+            )
+
     return res, dataset, fullpath, tmppath
 
 
@@ -439,11 +504,12 @@ def fetch_remote_setup_from_config(
     return terraform_working_dir, setup_type, setup
 
 
-def push_data_to_redistimeseries(rts: client, branch_time_series_dict: dict):
+def push_data_to_redistimeseries(rts, time_series_dict: dict):
+    logging.info(time_series_dict)
     datapoint_errors = 0
     datapoint_inserts = 0
-    if rts is not None:
-        for timeseries_name, time_series in branch_time_series_dict.items():
+    if rts is not None and time_series_dict is not None:
+        for timeseries_name, time_series in time_series_dict.items():
             exporter_create_ts(rts, time_series, timeseries_name)
             for timestamp, value in time_series["data"].items():
                 try:
@@ -474,7 +540,20 @@ def exporter_create_ts(rts, time_series, timeseries_name):
         )
         rts.create(timeseries_name, labels=time_series["labels"])
     except redis.exceptions.ResponseError:
-        logging.warning("Timeseries named {} already exists".format(timeseries_name))
+        logging.info(
+            "Timeseries named {} already exists. Checking that the labels match.".format(
+                timeseries_name
+            )
+        )
+        set1 = set(time_series["labels"].items())
+        set2 = set(rts.info(timeseries_name).labels.items())
+        if len(set1 - set2) > 0 or len(set2 - set1) > 0:
+            logging.info(
+                "Given the labels don't match using TS.ALTER on {} to update labels to {}".format(
+                    timeseries_name, time_series["labels"]
+                )
+            )
+            rts.alter(timeseries_name, labels=time_series["labels"])
         pass
 
 
@@ -493,18 +572,22 @@ def extract_perversion_timeseries_from_results(
     project_version: str,
     tf_github_org: str,
     tf_github_repo: str,
+    deployment_name: str,
     deployment_type: str,
     test_name: str,
     tf_triggering_env: str,
     metadata_tags={},
     build_variant_name=None,
+    running_platform=None,
+    testcase_metric_context_paths=[],
 ):
     break_by_key = "version"
     break_by_str = "by.{}".format(break_by_key)
-    branch_time_series_dict = common_timeseries_extraction(
+    (branch_time_series_dict, target_tables,) = common_timeseries_extraction(
         break_by_key,
         break_by_str,
         datapoints_timestamp,
+        deployment_name,
         deployment_type,
         metrics,
         project_version,
@@ -515,17 +598,20 @@ def extract_perversion_timeseries_from_results(
         tf_triggering_env,
         metadata_tags,
         build_variant_name,
+        running_platform,
+        testcase_metric_context_paths,
     )
-    return True, branch_time_series_dict
+    return True, branch_time_series_dict, target_tables
 
 
 def common_timeseries_extraction(
     break_by_key,
     break_by_str,
     datapoints_timestamp,
+    deployment_name,
     deployment_type,
     metrics,
-    project_version,
+    break_by_value,
     results_dict,
     test_name,
     tf_github_org,
@@ -533,85 +619,235 @@ def common_timeseries_extraction(
     tf_triggering_env,
     metadata_tags={},
     build_variant_name=None,
+    running_platform=None,
+    testcase_metric_context_paths=[],
 ):
-    branch_time_series_dict = {}
-    for jsonpath in metrics:
-        try:
-            jsonpath_expr = parse(jsonpath)
-        except Exception:
-            pass
-        finally:
-            find_res = jsonpath_expr.find(results_dict)
-            if find_res is not None:
-                use_metric_context_path = False
-                if len(find_res) > 1:
-                    use_metric_context_path = True
-                for metric in find_res:
-                    metric_name = str(metric.path)
-                    metric_value = float(metric.value)
-                    metric_jsonpath = jsonpath
-                    metric_context_path = str(metric.context.path)
-                    if metric_jsonpath[0] == "$":
-                        metric_jsonpath = metric_jsonpath[1:]
-                    if metric_jsonpath[0] == ".":
-                        metric_jsonpath = metric_jsonpath[1:]
+    time_series_dict = {}
+    target_tables = {}
+    cleaned_metrics_arr = extract_results_table(metrics, results_dict)
+    for cleaned_metric in cleaned_metrics_arr:
 
-                    # retro-compatible naming
-                    if use_metric_context_path is False:
-                        metric_name = metric_jsonpath
+        metric_jsonpath = cleaned_metric[0]
+        metric_context_path = cleaned_metric[1]
+        metric_name = cleaned_metric[2]
+        metric_value = cleaned_metric[3]
+        test_case_targets_dict = cleaned_metric[4]
+        use_metric_context_path = cleaned_metric[5]
 
-                    # prepare tags
-                    timeserie_tags = get_project_ts_tags(
-                        tf_github_org,
-                        tf_github_repo,
-                        deployment_type,
-                        tf_triggering_env,
-                    )
-                    timeserie_tags[break_by_key] = project_version
-                    timeserie_tags["test_name"] = str(test_name)
-                    timeserie_tags["metric"] = str(metric_name)
-                    timeserie_tags["metric_name"] = metric_name
-                    timeserie_tags["metric_context_path"] = metric_context_path
-                    timeserie_tags["metric_jsonpath"] = metric_jsonpath
-                    for k, v in metadata_tags.items():
-                        timeserie_tags[k] = v
-                    ts_name = get_ts_metric_name(
-                        break_by_str,
-                        project_version,
-                        tf_github_org,
-                        tf_github_repo,
-                        deployment_type,
-                        test_name,
-                        tf_triggering_env,
-                        metric_name,
-                        metric_context_path,
-                        use_metric_context_path,
-                        build_variant_name,
-                    )
-                    branch_time_series_dict[ts_name] = {
-                        "labels": timeserie_tags.copy(),
-                        "data": {datapoints_timestamp: metric_value},
-                    }
-            else:
-                logging.warning(
-                    "Unable to find metric path {} in result dict".format(jsonpath)
-                )
-    return branch_time_series_dict
+        target_table_keyname, target_table_dict = from_metric_kv_to_timeserie(
+            break_by_key,
+            break_by_str,
+            break_by_value,
+            build_variant_name,
+            datapoints_timestamp,
+            deployment_name,
+            deployment_type,
+            metadata_tags,
+            metric_context_path,
+            metric_jsonpath,
+            metric_name,
+            metric_value,
+            running_platform,
+            test_case_targets_dict,
+            test_name,
+            testcase_metric_context_paths,
+            tf_github_org,
+            tf_github_repo,
+            tf_triggering_env,
+            time_series_dict,
+            use_metric_context_path,
+        )
+        target_tables[target_table_keyname] = target_table_dict
+
+    return time_series_dict, target_tables
+
+
+def from_metric_kv_to_timeserie(
+    break_by_key,
+    break_by_str,
+    break_by_value,
+    build_variant_name,
+    datapoints_timestamp,
+    deployment_name,
+    deployment_type,
+    metadata_tags,
+    metric_context_path,
+    metric_jsonpath,
+    metric_name,
+    metric_value,
+    running_platform,
+    test_case_targets_dict,
+    test_name,
+    testcase_metric_context_paths,
+    tf_github_org,
+    tf_github_repo,
+    tf_triggering_env,
+    time_series_dict,
+    use_metric_context_path,
+):
+    timeserie_tags, ts_name = get_ts_tags_and_name(
+        break_by_key,
+        break_by_str,
+        break_by_value,
+        build_variant_name,
+        deployment_name,
+        deployment_type,
+        metadata_tags,
+        metric_context_path,
+        metric_jsonpath,
+        metric_name,
+        running_platform,
+        test_name,
+        testcase_metric_context_paths,
+        tf_github_org,
+        tf_github_repo,
+        tf_triggering_env,
+        use_metric_context_path,
+    )
+    time_series_dict[ts_name] = {
+        "labels": timeserie_tags.copy(),
+        "data": {datapoints_timestamp: metric_value},
+    }
+    original_ts_name = ts_name
+    target_table_keyname = "target_tables:{triggering_env}:ci.benchmarks.redislabs/{break_by_key}/{break_by_str}/{tf_github_org}/{tf_github_repo}/{deployment_type}/{deployment_name}/{test_name}/{metric_name}".format(
+        triggering_env=tf_triggering_env,
+        break_by_key=break_by_key,
+        break_by_str=break_by_str,
+        tf_github_org=tf_github_org,
+        tf_github_repo=tf_github_repo,
+        deployment_name=deployment_name,
+        deployment_type=deployment_type,
+        test_name=test_name,
+        metric_name=metric_name,
+    )
+    target_table_dict = {
+        "test-case": test_name,
+        "metric-name": metric_name,
+        tf_github_repo: metric_value,
+        "contains-target": False,
+    }
+    for target_name, target_value in test_case_targets_dict.items():
+        target_table_dict["contains-target"] = True
+        ts_name = original_ts_name + "/target/{}".format(target_name)
+        timeserie_tags_target = timeserie_tags.copy()
+        timeserie_tags_target["is_target"] = "true"
+        timeserie_tags_target["{}+{}".format("target", break_by_key)] = "{} {}".format(
+            break_by_value, target_name
+        )
+        time_series_dict[ts_name] = {
+            "labels": timeserie_tags_target,
+            "data": {datapoints_timestamp: target_value},
+        }
+        if "overallQuantiles" in metric_name:
+            comparison_type = "(lower-better)"
+        else:
+            comparison_type = "(higher-better)"
+        if comparison_type == "(higher-better)":
+            target_value_pct = (
+                (float(metric_value) / float(target_value)) - 1.0
+            ) * 100.0
+        else:
+            target_value_pct = (
+                (float(target_value) / float(metric_value)) - 1.0
+            ) * 100.0
+
+        target_value_pct_str = "{:.2f}".format(target_value_pct)
+
+        target_table_dict[target_name] = target_value
+
+        target_table_dict[
+            "{}:percent {}".format(target_name, comparison_type)
+        ] = target_value_pct_str
+    return target_table_keyname, target_table_dict
+
+
+def get_ts_tags_and_name(
+    break_by_key,
+    break_by_str,
+    break_by_value,
+    build_variant_name,
+    deployment_name,
+    deployment_type,
+    metadata_tags,
+    metric_context_path,
+    metric_jsonpath,
+    metric_name,
+    running_platform,
+    test_name,
+    testcase_metric_context_paths,
+    tf_github_org,
+    tf_github_repo,
+    tf_triggering_env,
+    use_metric_context_path,
+):
+    # prepare tags
+    timeserie_tags = get_project_ts_tags(
+        tf_github_org,
+        tf_github_repo,
+        deployment_name,
+        deployment_type,
+        tf_triggering_env,
+        metadata_tags,
+        build_variant_name,
+        running_platform,
+    )
+    timeserie_tags[break_by_key] = break_by_value
+    timeserie_tags["{}+{}".format("deployment_name", break_by_key)] = "{} {}".format(
+        deployment_name, break_by_value
+    )
+    timeserie_tags[break_by_key] = break_by_value
+    timeserie_tags["{}+{}".format("target", break_by_key)] = "{} {}".format(
+        break_by_value, tf_github_repo
+    )
+    timeserie_tags["test_name"] = str(test_name)
+    timeserie_tags["metric"] = str(metric_name)
+    timeserie_tags["metric_name"] = metric_name
+    timeserie_tags["metric_context_path"] = metric_context_path
+    timeserie_tags["metric_jsonpath"] = metric_jsonpath
+    if metric_context_path not in testcase_metric_context_paths:
+        testcase_metric_context_paths.append(metric_context_path)
+    ts_name = get_ts_metric_name(
+        break_by_str,
+        break_by_value,
+        tf_github_org,
+        tf_github_repo,
+        deployment_name,
+        deployment_type,
+        test_name,
+        tf_triggering_env,
+        metric_name,
+        metric_context_path,
+        use_metric_context_path,
+        build_variant_name,
+        running_platform,
+    )
+    return timeserie_tags, ts_name
 
 
 def get_project_ts_tags(
     tf_github_org: str,
     tf_github_repo: str,
+    deployment_name: str,
     deployment_type: str,
     tf_triggering_env: str,
+    metadata_tags={},
+    build_variant_name=None,
+    running_platform=None,
 ):
     tags = {
         "github_org": tf_github_org,
         "github_repo": tf_github_repo,
         "github_org/github_repo": "{}/{}".format(tf_github_org, tf_github_repo),
         "deployment_type": deployment_type,
+        "deployment_name": deployment_name,
         "triggering_env": tf_triggering_env,
     }
+    if build_variant_name is not None:
+        tags["build_variant"] = build_variant_name
+    if running_platform is not None:
+        tags["running_platform"] = running_platform
+    for k, v in metadata_tags.items():
+        tags[k] = str(v)
     return tags
 
 
@@ -622,18 +858,22 @@ def extract_perbranch_timeseries_from_results(
     tf_github_branch: str,
     tf_github_org: str,
     tf_github_repo: str,
+    deployment_name: str,
     deployment_type: str,
     test_name: str,
     tf_triggering_env: str,
     metadata_tags={},
     build_variant_name=None,
+    running_platform=None,
+    testcase_metric_context_paths=[],
 ):
     break_by_key = "branch"
     break_by_str = "by.{}".format(break_by_key)
-    branch_time_series_dict = common_timeseries_extraction(
+    (branch_time_series_dict, target_tables) = common_timeseries_extraction(
         break_by_key,
         break_by_str,
         datapoints_timestamp,
+        deployment_name,
         deployment_type,
         metrics,
         tf_github_branch,
@@ -644,12 +884,27 @@ def extract_perbranch_timeseries_from_results(
         tf_triggering_env,
         metadata_tags,
         build_variant_name,
+        running_platform,
+        testcase_metric_context_paths,
     )
-    return True, branch_time_series_dict
+    return True, branch_time_series_dict, target_tables
 
 
-def get_overall_dashboard_keynames(tf_github_org, tf_github_repo, tf_triggering_env):
-    prefix = (
+def get_overall_dashboard_keynames(
+    tf_github_org,
+    tf_github_repo,
+    tf_triggering_env,
+    build_variant_name=None,
+    running_platform=None,
+    test_name=None,
+):
+    build_variant_str = ""
+    if build_variant_name is not None:
+        build_variant_str = "/{}".format(build_variant_name)
+    running_platform_str = ""
+    if running_platform is not None:
+        running_platform_str = "/{}".format(running_platform)
+    sprefix = (
         "ci.benchmarks.redislabs/"
         + "{triggering_env}/{github_org}/{github_repo}".format(
             triggering_env=tf_triggering_env,
@@ -657,18 +912,54 @@ def get_overall_dashboard_keynames(tf_github_org, tf_github_repo, tf_triggering_
             github_repo=tf_github_repo,
         )
     )
-    testcases_setname = "{}:testcases".format(prefix)
+    testcases_setname = "{}:testcases".format(sprefix)
+    project_archs_setname = "{}:archs".format(sprefix)
+    project_oss_setname = "{}:oss".format(sprefix)
+    project_branches_setname = "{}:branches".format(sprefix)
+    project_versions_setname = "{}:versions".format(sprefix)
+    project_compilers_setname = "{}:compilers".format(sprefix)
+    running_platforms_setname = "{}:platforms".format(sprefix)
+    build_variant_setname = "{}:build_variants".format(sprefix)
+    build_variant_prefix = "{sprefix}{build_variant_str}".format(
+        sprefix=sprefix,
+        build_variant_str=build_variant_str,
+    )
+    prefix = "{build_variant_prefix}{running_platform_str}".format(
+        build_variant_prefix=build_variant_prefix,
+        running_platform_str=running_platform_str,
+    )
     tsname_project_total_success = "{}:total_success".format(
         prefix,
     )
     tsname_project_total_failures = "{}:total_failures".format(
         prefix,
     )
+    testcases_metric_context_path_setname = ""
+    if test_name is not None:
+        testcases_metric_context_path_setname = (
+            "{testcases_setname}:metric_context_path:{test_name}".format(
+                testcases_setname=testcases_setname, test_name=test_name
+            )
+        )
+    testcases_and_metric_context_path_setname = (
+        "{testcases_setname}_AND_metric_context_path".format(
+            testcases_setname=testcases_setname
+        )
+    )
     return (
         prefix,
         testcases_setname,
         tsname_project_total_failures,
         tsname_project_total_success,
+        running_platforms_setname,
+        build_variant_setname,
+        testcases_metric_context_path_setname,
+        testcases_and_metric_context_path_setname,
+        project_archs_setname,
+        project_oss_setname,
+        project_branches_setname,
+        project_versions_setname,
+        project_compilers_setname,
     )
 
 
